package com.isu.gaswellwatch.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.dao.DictionaryDao;
import com.isu.gaswellwatch.entity.Dictionary;
import com.isu.gaswellwatch.exception.BusinessException;
import com.isu.gaswellwatch.service.DictionaryService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service("dictionaryService")
@Transactional(rollbackFor = Exception.class)
public class DictionaryServiceImpl extends ServiceImpl<DictionaryDao, Dictionary> implements DictionaryService {

    @Resource
    private SnowflakeConfig snowflakeConfig;

    @Resource
    private DictionaryDao dictionaryDao;

    @Override
    public void add(Dictionary dictionary) {
        //查重  同一个类型下不能有重复code
        List<Dictionary> list = this.list(new LambdaQueryWrapper<Dictionary>().eq(Dictionary::getType, dictionary.getType())
                .eq(Dictionary::getCode, dictionary.getCode()));
        if (CollectionUtil.isNotEmpty(list)) {
            throw new BusinessException(dictionary.getType() + "类型下已有相同code，请重新输入");
        }
        Long dictionaryId = this.snowflakeConfig.snowflakeId();
        this.save(Dictionary.builder().id(dictionaryId).type(dictionary.getType()).code(dictionary.getCode()).name(dictionary.getName())
                .value(dictionary.getValue()).sort(dictionary.getSort()).build());
    }

    @Override
    public void edit(Dictionary dictionary) {
        //查重  同一个类型下不能有重复code
        List<Dictionary> list = this.list(new LambdaQueryWrapper<Dictionary>().eq(Dictionary::getType, dictionary.getType())
                .eq(Dictionary::getCode, dictionary.getCode()));
        if (CollectionUtil.isNotEmpty(list)) {
            throw new BusinessException(dictionary.getType() + "类型下已有相同code，请重新输入");
        }
        this.updateById(dictionary);
    }

    @Override
    public void delete(Long id) {
        //删除
        this.removeById(id);
    }

    @Override
    public List<Dictionary> getByType(String type) {
        return this.list(new LambdaQueryWrapper<Dictionary>().eq(Dictionary::getType, type).orderByAsc(Dictionary::getSort));
    }

    @Override
    public Dictionary getByTypeAndCode(String type, String code) {
        return this.getOne(new LambdaQueryWrapper<Dictionary>().eq(Dictionary::getType, type).eq(Dictionary::getCode, code));
    }

    @Override
    public Dictionary getByTypeAndValue(String type, String value) {
        return this.getOne(new LambdaQueryWrapper<Dictionary>().eq(Dictionary::getType, type).eq(Dictionary::getValue, value));
    }

    @Override
    public Map<String, Dictionary> getValueMapByType(String type) {
        return this.getByType(type).stream().collect(Collectors.toMap(Dictionary::getValue, a -> a, (k1, k2) -> k1));

    }


}

